==> murasaki_0.5.4.orig.tar.gz: murasaki-0.5.4/usb/match.c <==
/*
 *  Copyright (c) by Shuu Yamaguchi <shuu@dotaster.com>
 *
 *  $Id: match.c,v 3.6 2001/10/29 17:47:05 shuu Exp shuu $
 *
 *  Can be freely distributed and used under the terms of the GNU GPL.
 */
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#if 0	/* need __KERNEL__ */
#include	<linux/usb.h>
#else
#define USB_DEVICE_ID_MATCH_VENDOR      0x0001
#define USB_DEVICE_ID_MATCH_PRODUCT     0x0002
#define USB_DEVICE_ID_MATCH_DEV_LO      0x0004
#define USB_DEVICE_ID_MATCH_DEV_HI      0x0008
#define USB_DEVICE_ID_MATCH_DEV_CLASS       0x0010
#define USB_DEVICE_ID_MATCH_DEV_SUBCLASS    0x0020
#define USB_DEVICE_ID_MATCH_DEV_PROTOCOL    0x0040
#define USB_DEVICE_ID_MATCH_INT_CLASS       0x0080
#define USB_DEVICE_ID_MATCH_INT_SUBCLASS    0x0100
#define USB_DEVICE_ID_MATCH_INT_PROTOCOL    0x0200
#endif

#include	"murasaki.h"
#include	"env.h"

#define	CHAR_COMMENT	'#'
#define	CHAR_RSEP		'\n'

#define	ST_COMMENT		255
#define	ST_SKIP			(ST_COMMENT - 1)
#define	ST_MODULE		1
#define	ST_MATCH_FLAGS	(ST_MODULE + 1)
#define	ST_VENDOR		(ST_MATCH_FLAGS + 1)
#define	ST_PRODUCT		(ST_VENDOR + 1)
#define	ST_DEV_LO		(ST_PRODUCT + 1)
#define	ST_DEV_HI		(ST_DEV_LO + 1)
#define	ST_DCLASS		(ST_DEV_HI + 1)
#define	ST_DSUBCLASS	(ST_DCLASS + 1)
#define	ST_DPROTOCOL	(ST_DSUBCLASS + 1)
#define	ST_ICLASS		(ST_DPROTOCOL + 1)
#define	ST_ISUBCLASS	(ST_ICLASS + 1)
#define	ST_IPROTOCOL	(ST_ISUBCLASS + 1)
#define	ST_INFO			(ST_IPROTOCOL + 1)

static int
real_match(MU_usb_config_t *active,MU_usb_config_t *map)
{
	if (active->product.vendor == 0 &&
		active->device.class == 0 &&
		active->interface.class == 0)
		return INVALID;
	if (map->match_flags == 0) /* umm... */
		return INVALID;
#ifdef	DEBUG_ALL
	DPRINTF(LOG_LEVEL,"match_flags:0x%x vendor:0x%x product:0x%x Dclass:0x%x Dsubclass:0x%x Dprotocol:0x%x Iclass:0x%x Isubclass:0x%x Iprotocol:0x%x",
	map->match_flags,
	map->product.vendor, map->product.product,
	map->device.class, map->device.subclass,
	map->device.protocol,
	map->interface.class, map->interface.subclass,
	map->interface.protocol);
#endif

	/* Vendor */
	if ((map->match_flags & USB_DEVICE_ID_MATCH_VENDOR) &&
		(active->product.vendor != map->product.vendor))
		return INVALID;
	if ((map->match_flags & USB_DEVICE_ID_MATCH_PRODUCT) &&
		(active->product.product != map->product.product))
		return INVALID;
	/* bsdDevice value in the member device_lo */
	if ((map->match_flags & USB_DEVICE_ID_MATCH_DEV_LO) &&
		(active->product.device_lo < map->product.device_lo))
		return INVALID;
	if ((map->match_flags & USB_DEVICE_ID_MATCH_DEV_HI) &&
		(active->product.device_lo > map->product.device_hi))
		return INVALID;

	/* DeviceClass */
	if ((map->match_flags & USB_DEVICE_ID_MATCH_DEV_CLASS) &&
		(active->device.class != map->device.class))
		return INVALID;
	if ((map->match_flags & USB_DEVICE_ID_MATCH_DEV_SUBCLASS) &&
		(active->device.subclass != map->device.subclass))
		return INVALID;
	if ((map->match_flags & USB_DEVICE_ID_MATCH_DEV_PROTOCOL) &&
		(active->device.protocol != map->device.protocol))
		return INVALID;

	/* InterfaceClass */
	if ((map->match_flags & USB_DEVICE_ID_MATCH_INT_CLASS) &&
		(active->interface.class != map->interface.class))
		return INVALID;
	if ((map->match_flags & USB_DEVICE_ID_MATCH_INT_SUBCLASS) &&
		(active->interface.subclass != map->interface.subclass))
		return INVALID;
	if ((map->match_flags & USB_DEVICE_ID_MATCH_INT_PROTOCOL) &&
		(active->interface.protocol != map->interface.protocol))
		return INVALID;

	return GOOD;
}

int
match_config(char **list,char *start,char *end,void *conf)
{
	MU_usb_config_t *config = conf;
	MU_usb_config_t map;
	char *ptr,*module,*module_start,*module_end = NULL;
	int status = ST_MODULE;
	unsigned int len,value;
	char *fwdp;
	int mod_count = 0;

	DPRINTF(LOG_LEVEL,"vendor:0x%x product:0x%x device:0x%x Dclass:0x%x Dsubclass:0x%x Dprotocol:0x%x Iclass:0x%x Isubclass:0x%x Iprotocol:0x%x",
		config->product.vendor, config->product.product,
		config->product.device_lo,
		config->device.class, config->device.subclass,
		config->device.protocol,
		config->interface.class, config->interface.subclass,
		config->interface.protocol);
	for(module_start = ptr = start; ptr < end;ptr++) {
		if (status == ST_COMMENT) {
			while(*ptr != CHAR_RSEP) {
				if (ptr >= end)
					goto error_return;
				ptr++;
			}
			status = ST_MODULE;
			continue;
		} else if (*ptr == CHAR_COMMENT) {
			status = ST_COMMENT;
			continue;
		}
		if (status == ST_MODULE) {
			module_start = ptr;
			while(*ptr != ' ' && *ptr != '\t') {
				if (ptr >= end)
					goto error_return;
				ptr++;
			}
			module_end = ptr;
		} else {
			value = (unsigned int)strtoul(ptr,&fwdp,0);
			ptr = fwdp;
			switch(status) {
			case ST_MATCH_FLAGS:
				map.match_flags = value;
				break;
			case ST_VENDOR:
				map.product.vendor = value;
				break;
			case ST_PRODUCT:
				map.product.product = value;
				break;
			case ST_DEV_LO:
				map.product.device_lo = value;
				break;
			case ST_DEV_HI:
				map.product.device_hi = value;
				break;
			case ST_DCLASS:
				map.device.class = value;
				break;
			case ST_DSUBCLASS:
				map.device.subclass = value;
				break;
			case ST_DPROTOCOL:
				map.device.protocol = value;
				break;
			case ST_ICLASS:
				map.interface.class = value;
				break;
			case ST_ISUBCLASS:
				map.interface.subclass = value;
				break;
			case ST_IPROTOCOL:
				map.interface.protocol = value;
				break;
			case ST_INFO:		/* ignore */
				status = ST_MODULE;
				if (real_match(config,&map) == GOOD) {
					len = module_end - module_start;
					module = malloc(len+1);
					if (module == NULL)
						goto error_return;
					memcpy(module,module_start,len);
					module[len] = '\0';

					syslog(LOG_LEVEL,"MATCH(%s) -> match_flags:0x%x vendor:0x%x product:0x%x Dclass:0x%x Dsubclass:0x%x Dprotocol:0x%x Iclass:0x%x Isubclass:0x%x Iprotocol:0x%x",
					module,map.match_flags,
					map.product.vendor,map.product.product,
					map.device.class,map.device.subclass,map.device.protocol,
					map.interface.class,map.interface.subclass,
					map.interface.protocol);

					if (add_to_list(list,module,LIST_MAX) == INVALID)
						goto error_return;
					mod_count++;
				}
				continue;
			default:
				goto error_return;
			}
		}
		status++;
	}
	return mod_count;

error_return:
	return INVALID;
}

==> murasaki-0.7.12.tar.gz: murasaki-0.7.12/usb/match.c <==
/*
 *  Copyright (c) by Shuu Yamaguchi <shuu@dotaster.com>
 *
 *  $Id: match.c,v 3.9 2003/08/10 04:23:10 shuu Exp shuu $
 *
 *  Can be freely distributed and used under the terms of the GNU GPL.
 */
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

#include	"murasaki.h"
#include	"murasaki_usb.h"

#define	CHAR_COMMENT	'#'
#define	CHAR_RSEP		'\n'

#define	ST_COMMENT		255
#define	ST_SKIP			(ST_COMMENT - 1)
#define	ST_MODULE		1
#define	ST_MATCH_FLAGS	(ST_MODULE + 1)
#define	ST_VENDOR		(ST_MATCH_FLAGS + 1)
#define	ST_PRODUCT		(ST_VENDOR + 1)
#define	ST_DEV_LO		(ST_PRODUCT + 1)
#define	ST_DEV_HI		(ST_DEV_LO + 1)
#define	ST_DCLASS		(ST_DEV_HI + 1)
#define	ST_DSUBCLASS	(ST_DCLASS + 1)
#define	ST_DPROTOCOL	(ST_DSUBCLASS + 1)
#define	ST_ICLASS		(ST_DPROTOCOL + 1)
#define	ST_ISUBCLASS	(ST_ICLASS + 1)
#define	ST_IPROTOCOL	(ST_ISUBCLASS + 1)
#define	ST_INFO			(ST_IPROTOCOL + 1)

static int
real_match(MU_usb_config_t *active,MU_usb_config_t *map)
{
	if (active->product.vendor == 0 &&
		active->device.class == 0 &&
		active->interface.class == 0)
		return INVALID;
	if (map->match_flags == 0) /* umm... */
		return INVALID;
#ifdef	DEBUG_ALL
	DPRINTF(LOG_LEVEL,"match_flags:0x%x vendor:0x%x product:0x%x Dclass:0x%x Dsubclass:0x%x Dprotocol:0x%x Iclass:0x%x Isubclass:0x%x Iprotocol:0x%x",
	map->match_flags,
	map->product.vendor, map->product.product,
	map->device.class, map->device.subclass,
	map->device.protocol,
	map->interface.class, map->interface.subclass,
	map->interface.protocol);
#endif

	/* Vendor */
	if ((map->match_flags & USB_DEVICE_ID_MATCH_VENDOR) &&
		(active->product.vendor != map->product.vendor))
		return INVALID;
	if ((map->match_flags & USB_DEVICE_ID_MATCH_PRODUCT) &&
		(active->product.product != map->product.product))
		return INVALID;
	/* bsdDevice value in the member device_lo */
	if ((map->match_flags & USB_DEVICE_ID_MATCH_DEV_LO) &&
		(active->product.device_lo < map->product.device_lo))
		return INVALID;
	if ((map->match_flags & USB_DEVICE_ID_MATCH_DEV_HI) &&
		(active->product.device_lo > map->product.device_hi))
		return INVALID;

	/* DeviceClass */
	if ((map->match_flags & USB_DEVICE_ID_MATCH_DEV_CLASS) &&
		(active->device.class != map->device.class))
		return INVALID;
	if ((map->match_flags & USB_DEVICE_ID_MATCH_DEV_SUBCLASS) &&
		(active->device.subclass != map->device.subclass))
		return INVALID;
	if ((map->match_flags & USB_DEVICE_ID_MATCH_DEV_PROTOCOL) &&
		(active->device.protocol != map->device.protocol))
		return INVALID;

	/* InterfaceClass */
	if ((map->match_flags & USB_DEVICE_ID_MATCH_INT_CLASS) &&
		(active->interface.class != map->interface.class))
		return INVALID;
	if ((map->match_flags & USB_DEVICE_ID_MATCH_INT_SUBCLASS) &&
		(active->interface.subclass != map->interface.subclass))
		return INVALID;
	if ((map->match_flags & USB_DEVICE_ID_MATCH_INT_PROTOCOL) &&
		(active->interface.protocol != map->interface.protocol))
		return INVALID;

	return GOOD;
}

int
match_config(char **list,char **except_list,char *start,char *end,void *conf)
{
	MU_usb_config_t *config = conf;
	MU_usb_config_t map;
	char *ptr,*module,*module_start,*module_end = NULL;
	int status = ST_MODULE;
	unsigned int module_len = 0,value,i;
	char *fwdp;
	int mod_count = 0;

	DPRINTF(LOG_LEVEL,"vendor:0x%x product:0x%x device:0x%x Dclass:0x%x Dsubclass:0x%x Dprotocol:0x%x Iclass:0x%x Isubclass:0x%x Iprotocol:0x%x",
		config->product.vendor, config->product.product,
		config->product.device_lo,
		config->device.class, config->device.subclass,
		config->device.protocol,
		config->interface.class, config->interface.subclass,
		config->interface.protocol);
	for(module_start = ptr = start; ptr < end;ptr++) {
		if (status == ST_COMMENT) {
			while(*ptr != CHAR_RSEP) {
				if (ptr >= end)
					goto error_return;
				ptr++;
			}
			status = ST_MODULE;
			continue;
		} else if (*ptr == CHAR_COMMENT) {
			status = ST_COMMENT;
			continue;
		}
		if (status == ST_MODULE) {
			module_start = ptr;
			while(*ptr != ' ' && *ptr != '\t') {
				if (ptr >= end)
					goto error_return;
				ptr++;
			}
			module_end = ptr;
			module_len = module_end - module_start;
			/*
			 * check excepted modules
			 */
			for (i = 0;except_list[i] != NULL;i++) {
				if (memcmp(except_list[i],module_start,module_len) == 0 && except_list[i][module_len] == '\0') {
					break;
				}
			}
			if (except_list[i] != NULL) {
				status = ST_COMMENT;
				syslog(LOG_LEVEL,"%s was skipped in usbmap",except_list[i]);
				continue;
			}
		} else {
			value = (unsigned int)strtoul(ptr,&fwdp,0);
			ptr = fwdp;
			switch(status) {
			case ST_MATCH_FLAGS:
				map.match_flags = value;
				break;
			case ST_VENDOR:
				map.product.vendor = value;
				break;
			case ST_PRODUCT:
				map.product.product = value;
				break;
			case ST_DEV_LO:
				map.product.device_lo = value;
				break;
			case ST_DEV_HI:
				map.product.device_hi = value;
				break;
			case ST_DCLASS:
				map.device.class = value;
				break;
			case ST_DSUBCLASS:
				map.device.subclass = value;
				break;
			case ST_DPROTOCOL:
				map.device.protocol = value;
				break;
			case ST_ICLASS:
				map.interface.class = value;
				break;
			case ST_ISUBCLASS:
				map.interface.subclass = value;
				break;
			case ST_IPROTOCOL:
				map.interface.protocol = value;
				break;
			case ST_INFO:		/* ignore */
				status = ST_MODULE;
				/* for exclusive */
				if (config->match_flags &&
					config->match_flags == map.match_flags)
					continue;
				if (real_match(config,&map) == GOOD) {
					/* for exclusive */
					config->match_flags = map.match_flags;
					module = malloc(module_len+1);
					if (module == NULL)
						goto error_return;
					memcpy(module,module_start,module_len);
					module[module_len] = '\0';

					syslog(LOG_LEVEL,"MATCH(%s) -> match_flags:0x%x vendor:0x%x product:0x%x Dclass:0x%x Dsubclass:0x%x Dprotocol:0x%x Iclass:0x%x Isubclass:0x%x Iprotocol:0x%x",
					module,map.match_flags,
					map.product.vendor,map.product.product,
					map.device.class,map.device.subclass,map.device.protocol,
					map.interface.class,map.interface.subclass,
					map.interface.protocol);

					if (add_to_list(list,module,LIST_MAX) == INVALID)
						goto error_return;
					mod_count++;
				}
				continue;
			default:
				goto error_return;
			}
		}
		status++;
	}
	return mod_count;

error_return:
	return INVALID;
}
